@@ -13,18 +13,18 @@
  * documentation.  This software is provided "as is" without expressed or
  * implied warranty.
 
- * Creation: Mon Mar 21 00:52:43 GMT 1994 on mohegan
+ * Creation: Sun Jun 19 17:43:25 GMT 1994 on mohegan
  * Author: genc
- * Version: 0.95999999999999996447
+ * Version: 0.96999999999999997335
+
 
  */
 #define MISC
 
+#define MISC_C
+#include <wafe.h>
 #ifdef MISC
 
-#define MISC_C
-
-#include "wafe.h"
 #include <time.h>
 #define ref(a) &(a)
 
@@ -100,13 +100,12 @@ String oldCmdName;
     XtPointer cmdPtr;
     Interp *iPtr = (Interp *) wafeInterpreter;
     Tcl_HashEntry *hPtr;
-    int new;    
+    int newEntry;    
 
     if ((hPtr = Tcl_FindHashEntry(&iPtr->commandTable, newCmdName)) != NULL)
 	{
-	fprintf(stderr,
-		"Wafe(alias): can't alias to '%s': command already exists\n",
-		newCmdName);
+	wafeWarn("alias","can't alias to '%s': command already exists",
+		 newCmdName, NULL,NULL);
         /* Tcl_AppendResult(wafeInterpreter, "can't alias to \"", newCmdName,
                 "\": command already exists", (char *) NULL); */
         return 0;
@@ -114,16 +113,15 @@ String oldCmdName;
     
     if ((hPtr = Tcl_FindHashEntry(&iPtr->commandTable, oldCmdName)) == NULL)
 	{
-	fprintf(stderr,
-		"Wafe(alias): can't alias '%s': command doesn't exist\n",
-		oldCmdName);
+	wafeWarn("alias","can't alias '%s': command doesn't exist",
+		 oldCmdName, NULL,NULL);
         /* Tcl_AppendResult(wafeInterpreter, "can't alias \"", oldCmdName,
                 "\":  command doesn't exist", (char *) NULL); */
         return 0;
 	}
 
     cmdPtr = (XtPointer) Tcl_GetHashValue(hPtr);
-    hPtr = Tcl_CreateHashEntry(&iPtr->commandTable, newCmdName, &new);
+    hPtr = Tcl_CreateHashEntry(&iPtr->commandTable, newCmdName, &newEntry);
     Tcl_SetHashValue(hPtr, cmdPtr);
     return 1;
 }
@@ -147,17 +145,17 @@ char        **argv;
      time_t localVar1;
      String returnVar;
 
-     DBUG_ENTER("ctime");  
+     DBUG_ENTER(argv[0]);  
 
      if (argc != 2) 
 	 {
-	 wafeArgcError("ctime","",1,argc);
+	 wafeArgcError(argv[0],"",1,argc);
 	 DBUG_RETURN (TCL_ERROR);
          }
 
      if (!(sscanf(argv[1], "%ld", &localVar1)))
           {
-          wafeConvError("ctime","1",argv[1],"time_t");
+          wafeConvError(argv[0],"1",argv[1],"time_t");
           DBUG_RETURN (TCL_ERROR);
           }
 
@@ -190,11 +188,11 @@ char        **argv;
      String localVar1;
      int count;
 
-     DBUG_ENTER("echo");  
+     DBUG_ENTER(argv[0]);  
 
      if (argc < 2) 
 	 {
-	 wafeArgcError("echo","at least ",1,argc);
+	 wafeArgcError(argv[0],"at least ",1,argc);
 	 DBUG_RETURN (TCL_ERROR);
          }
 
@@ -227,11 +225,11 @@ char        **argv;
      int returnVar;
      char conversionBuffer[100];
 
-     DBUG_ENTER("getChannel");  
+     DBUG_ENTER(argv[0]);  
 
      if (argc != 1) 
 	 {
-	 wafeArgcError("getChannel","",0,argc);
+	 wafeArgcError(argv[0],"",0,argc);
 	 DBUG_RETURN (TCL_ERROR);
          }
 
@@ -252,7 +250,7 @@ char        **argv;
  * 	const: wafeClientPid	
  * 	const: SIGTERM
  * 	pre: if (wafeClientPid)		# None
- * 	post: exit(0);
+ * 	post: wafeExit(0);
  */
 
 static int 
@@ -263,17 +261,17 @@ int           argc;
 char        **argv;
      {
 
-     DBUG_ENTER("quit");  
+     DBUG_ENTER(argv[0]);  
 
      if (argc != 1) 
 	 {
-	 wafeArgcError("quit","",0,argc);
+	 wafeArgcError(argv[0],"",0,argc);
 	 DBUG_RETURN (TCL_ERROR);
          }
 
      if (wafeClientPid)
      kill(wafeClientPid,SIGTERM);
-     exit(0);
+     wafeExit(0);
 
      DBUG_RETURN (TCL_OK);
      }     
@@ -297,17 +295,17 @@ char        **argv;
      int localVar1;
      int localVar2;
 
-     DBUG_ENTER("kill");  
+     DBUG_ENTER(argv[0]);  
 
      if (argc != 3) 
 	 {
-	 wafeArgcError("kill","",2,argc);
+	 wafeArgcError(argv[0],"",2,argc);
 	 DBUG_RETURN (TCL_ERROR);
          }
 
      if (!(sscanf(argv[1], "%d", &localVar1)))
           {
-          wafeConvError("kill","1",argv[1],"int");
+          wafeConvError(argv[0],"1",argv[1],"int");
           DBUG_RETURN (TCL_ERROR);
           }
 
@@ -361,11 +359,11 @@ char        **argv;
      {
      int localVar1;
 
-     DBUG_ENTER("register");  
+     DBUG_ENTER(argv[0]);  
 
      if (argc != 3) 
 	 {
-	 wafeArgcError("register","",2,argc);
+	 wafeArgcError(argv[0],"",2,argc);
 	 DBUG_RETURN (TCL_ERROR);
          }
 
@@ -436,11 +434,11 @@ char        **argv;
      {
      int localVar2;
 
-     DBUG_ENTER("setCommunicationVariable");  
+     DBUG_ENTER(argv[0]);  
 
      if (argc != 4) 
 	 {
-	 wafeArgcError("setCommunicationVariable","",3,argc);
+	 wafeArgcError(argv[0],"",3,argc);
 	 DBUG_RETURN (TCL_ERROR);
          }
 
@@ -448,7 +446,7 @@ char        **argv;
 
      if (!(sscanf(argv[2], "%d", &localVar2)))
           {
-          wafeConvError("setCommunicationVariable","2",argv[2],"int");
+          wafeConvError(argv[0],"2",argv[2],"int");
           DBUG_RETURN (TCL_ERROR);
           }
 
@@ -475,11 +473,11 @@ char        **argv;
      {
      int localVar1;
 
-     DBUG_ENTER("unregister");  
+     DBUG_ENTER(argv[0]);  
 
      if (argc != 2) 
 	 {
-	 wafeArgcError("unregister","",1,argc);
+	 wafeArgcError(argv[0],"",1,argc);
 	 DBUG_RETURN (TCL_ERROR);
          }
 
@@ -552,17 +550,17 @@ char        **argv;
      XtInputId returnVar;
      char conversionBuffer[100];
 
-     DBUG_ENTER("addInput");  
+     DBUG_ENTER(argv[0]);  
 
      if (argc != 5) 
 	 {
-	 wafeArgcError("addInput","",4,argc);
+	 wafeArgcError(argv[0],"",4,argc);
 	 DBUG_RETURN (TCL_ERROR);
          }
 
      if (!(sscanf(argv[1], "%d", &localVar1)))
           {
-          wafeConvError("addInput","1",argv[1],"int");
+          wafeConvError(argv[0],"1",argv[1],"int");
           DBUG_RETURN (TCL_ERROR);
           }
 
@@ -612,17 +610,17 @@ char        **argv;
      {
      XtInputId localVar1;
 
-     DBUG_ENTER("removeInput");  
+     DBUG_ENTER(argv[0]);  
 
      if (argc != 2) 
 	 {
-	 wafeArgcError("removeInput","",1,argc);
+	 wafeArgcError(argv[0],"",1,argc);
 	 DBUG_RETURN (TCL_ERROR);
          }
 
      if (!(sscanf(argv[1], "%lu", &localVar1)))
           {
-          wafeConvError("removeInput","1",argv[1],"XtInputId");
+          wafeConvError(argv[0],"1",argv[1],"XtInputId");
           DBUG_RETURN (TCL_ERROR);
           }
 
@@ -654,11 +652,11 @@ char        **argv;
      int returnVar;
      char conversionBuffer[100];
 
-     DBUG_ENTER("fileno");  
+     DBUG_ENTER(argv[0]);  
 
      if (argc != 3) 
 	 {
-	 wafeArgcError("fileno","",2,argc);
+	 wafeArgcError(argv[0],"",2,argc);
 	 DBUG_RETURN (TCL_ERROR);
          }
 
@@ -710,11 +708,11 @@ char        **argv;
      int returnVar;
      char conversionBuffer[100];
 
-     DBUG_ENTER("alias");  
+     DBUG_ENTER(argv[0]);  
 
      if (argc != 3) 
 	 {
-	 wafeArgcError("alias","",2,argc);
+	 wafeArgcError(argv[0],"",2,argc);
 	 DBUG_RETURN (TCL_ERROR);
          }
 
@@ -731,8 +729,65 @@ char        **argv;
      }     
 
 
+/* 
+ * void
+ * DBUG_PUSH
+ * 	in: String			# new debug options
+ */
+
+static int 
+cmd_DBUG_PUSH(clientData, comInterpreter, argc, argv)
+ClientData    clientData;
+Tcl_Interp   *comInterpreter;
+int           argc;
+char        **argv;
+     {
+
+     DBUG_ENTER(argv[0]);  
+
+     if (argc != 2) 
+	 {
+	 wafeArgcError(argv[0],"",1,argc);
+	 DBUG_RETURN (TCL_ERROR);
+         }
+
+    /* no need to assign  << localVar1 = argv[1] >>  */ 
+
+     DBUG_PUSH(argv[1]);
+
+     DBUG_RETURN (TCL_OK);
+     }     
+
+
+/* 
+ * void
+ * DBUG_POP 
+
+ */
+
+static int 
+cmd_DBUG_POP(clientData, comInterpreter, argc, argv)
+ClientData    clientData;
+Tcl_Interp   *comInterpreter;
+int           argc;
+char        **argv;
+     {
+
+     DBUG_ENTER(argv[0]);  
+
+     if (argc != 1) 
+	 {
+	 wafeArgcError(argv[0],"",0,argc);
+	 DBUG_RETURN (TCL_ERROR);
+         }
+
+     DBUG_POP();
+
+     DBUG_RETURN (TCL_OK);
+     }     
+
+
 #else
-#include "wafe.h"
 #endif
 
 void
@@ -751,6 +806,8 @@ wafeInitialize_MiscGen()
  Tcl_CreateCommand(wafeInterpreter, "removeInput", cmd_removeInput, NULL, NULL);
  Tcl_CreateCommand(wafeInterpreter, "fileno", cmd_fileno, NULL, NULL);
  Tcl_CreateCommand(wafeInterpreter, "alias", cmd_alias, NULL, NULL);
+ Tcl_CreateCommand(wafeInterpreter, "DBUG_PUSH", cmd_DBUG_PUSH, NULL, NULL);
+ Tcl_CreateCommand(wafeInterpreter, "DBUG_POP", cmd_DBUG_POP, NULL, NULL);
 
 #else
 #endif
