@@ -376,7 +376,7 @@ rddSetSelection(data, nbytes)
 	}
 
 	rddSelnData = (u_char *)XtMalloc(rddSelnLen=nbytes);
-	bcopy (data, rddSelnData, rddSelnLen);
+	memcpy(rddSelnData, data, rddSelnLen);
 
 	stat = XChangeProperty (display, RootWindow(display,0), rddSelnAtom,
 							XA_STRING,
