@@ -16,7 +16,7 @@
  *
  * Date: Mon, Apr 13 1992
  * Author: Stefan Nusser, Gustaf Neumann
- * Version: 0.96
+ * Version: 0.97
  */
 
 /*
@@ -28,25 +28,29 @@
 
 #define MAIN
 
-#include <sys/types.h>
 #include <ctype.h>
 #include "wafe.h"
-#include <X11/Xos.h>
 
 #ifndef STREAMS
-#include <sys/socket.h>
+# define SOCKET_CONNECTION
+# include <sys/socket.h>
+# ifdef SOCKET_CONNECTION
+#  include <netinet/in.h>
+#  include <sys/un.h>
+#  include <netdb.h>
+# endif
 #else
-#include <sys/stream.h>
-#include <stropts.h>
-#include <fcntl.h>
+# include <sys/stream.h>
+# include <stropts.h>
 #endif
 
 #ifdef RDD
-#include <rdd.h>
+# include <rdd.h>
 #endif
 
 #include "version.h"
 
+#define wafePerror(msg) {perror(msg); wafeExit(-1);}
 /*****************************************************************************
  *    FUNCTION: iSocketPair
  *            Use socketpair on BSD or ioctl on Sys 5 or pipes
@@ -130,18 +134,6 @@ int      fd[4], onlyOne;
 #endif
 #endif
 
-void
-wafeError(part,message, arg) 
-String part;
-String message;
-String arg;
-    {
-    fprintf(stderr,"Wafe(%s): ",part);
-    fprintf(stderr,message,arg);
-    fprintf(stderr, "; aborting...\n");
-    exit(-1);
-    }
-
 static char *
 checkArg(argc,args, argv, c)
 int argc;
@@ -158,13 +150,112 @@ char c;
 	char buffer[2];
 	buffer[0] = c;
 	buffer[1] = 0;
-        wafeError("main", "Option --%s needs an additional argument", buffer);
+        wafeFatal("main", "Option --%s needs an additional argument", buffer);
         }
 
     return argv[*args];
     }
 
+#ifdef SOCKET_CONNECTION
+#define SADDR(x) (struct sockaddr *) &x
+static int
+unixStreamConnection(socketName)
+char *socketName;
+    {
+    struct sockaddr_un s;
+    int servlen, fd, nfd;
+    struct sockaddr local;
+    int len = sizeof(local);
 
+    DBUG_PRINT("socket",("will use unix domain socket %s",socketName));
+    if ((fd = socket(AF_UNIX, SOCK_STREAM, 0)) < 0) wafePerror("socket");
+    bzero((char*)&s, sizeof(s));
+    s.sun_family = AF_UNIX;
+    strcpy(s.sun_path,socketName);
+    servlen = strlen(s.sun_path) + 1 + sizeof(s.sun_family);
+    if (bind(fd, SADDR(s), servlen) < 0) wafePerror("bind");
+    if (listen(fd,1) != 0) 
+	{
+	unlink(socketName);  
+	wafePerror("listen");  
+	}
+    if ((nfd = accept(fd, &local, &len)) < 0) 
+	{
+	unlink(socketName);  
+	wafePerror("accept");
+	}
+    unlink(socketName);  /*the socket will exist until the program terminates*/
+    close(fd);
+    return nfd;
+    }
+
+static int
+tcpPortConnection(port)
+int port;
+    {
+    struct sockaddr_in s,client;
+    int fd, nfd, len = sizeof(client);
+    DBUG_PRINT("socket",("accepting connections on port %d",port));
+
+    if ((fd = socket(AF_INET, SOCK_STREAM, 0)) < 0) wafePerror("socket");
+    bzero((char*)&s, sizeof(s));
+    s.sin_family      = AF_INET;
+    s.sin_addr.s_addr = htonl(INADDR_ANY);
+    s.sin_port        = htons(port);
+    if (bind(fd, SADDR(s), sizeof(s)) < 0) wafePerror("bind");
+    if (listen(fd,1) != 0) wafePerror("listen");  
+    if ((nfd = accept(fd, SADDR(client), &len)) < 0)  wafePerror("accept");
+    close(fd);
+    return nfd;
+    }
+
+static int
+tcpHostPortConnection(host,port)
+char *host;
+int port;
+    {
+    struct sockaddr_in s,client;
+    int fd0, fd, nfd, len = sizeof(client);
+    struct hostent* h = gethostbyname(host);
+    char portString[INT_AS_STRING];
+    
+    DBUG_PRINT("socket",("trying connection to host %s port %d",host,port));
+    h = gethostbyname(host);
+    if (h == NULL || h->h_addrtype != AF_INET) wafePerror("gethostbyname");
+    
+    /* connect to server on specified host and port */
+    if ((fd0 = socket(AF_INET, SOCK_STREAM, 0)) < 0) wafePerror("socket");
+    bzero((char*)&s, sizeof(s));
+    s.sin_family = AF_INET;
+    memcpy((char *)&s.sin_addr,(char *)h->h_addr,h->h_length);
+    s.sin_port = htons(port);
+
+    DBUG_PRINT("socket",("trying to connect to %s:%d",host,port));
+    if (connect(fd0, SADDR(s), sizeof(s)) < 0) wafePerror("connect");
+
+    /* create new temporary port and play server on this port */    
+    if ((fd = socket(AF_INET, SOCK_STREAM, 0)) < 0) wafePerror("socket");
+    bzero((char*)&s, sizeof(s));
+    s.sin_family      = AF_INET;
+    s.sin_addr.s_addr = htonl(INADDR_ANY);
+    s.sin_port        = 0;
+    if (bind(fd, SADDR(s), sizeof(s)) < 0) wafePerror("bind");
+    if (getsockname(fd, SADDR(s), &len)<0) wafePerror("getsockname");
+    DBUG_PRINT("socket",("listening on new port %d",ntohs(s.sin_port)));
+    if (listen(fd,1) != 0) wafePerror("listen");  
+
+    /* tell partner the new port in form of a string */    
+    sprintf(portString, "%d", ntohs(s.sin_port));
+    write(fd0, portString, strlen(portString)+1);
+    close(fd0);
+    DBUG_PRINT("socket",("new port address sent, waiting for connection..."));
+
+    /* accept connections on the new port */    
+    if ((nfd = accept(fd, SADDR(client), &len)) < 0) wafePerror("accept");
+    close(fd);
+    return nfd;
+    }
+#endif
 
 /*****************************************************************************
  *    FUNCTION:  MAIN                                
@@ -198,6 +289,11 @@ char **env;
     char      *charPtr;
     char      *execString = NULL;
     char      *applicationName = NULL;
+#ifdef SOCKET_CONNECTION
+    int        port = 0;
+    char      *hostName = NULL;
+    char      *socketName = NULL;
+#endif
     int        sp[4];  
     int        ep[4];  
     int        cp[4];
@@ -295,7 +391,7 @@ char **env;
 	        {
 		charPtr = checkArg(argc,&args, argv, flagChar);
 		if (!(sscanf(charPtr,"%d/%d",&wafeExtraCom,&wafeExtraClient)))
-		    wafeError("main", 
+		    wafeFatal("main", 
 			      "Option --t needs fileno(wafeSock)/fileno(clientSock)", NULL);
                 }
             else
@@ -308,6 +404,29 @@ char **env;
 		explicitClassName = True;
                 }
             else
+
+#ifdef SOCKET_CONNECTION
+	    if (flagChar == 'S')   /*** Socket connection ***/
+	        {
+		char *colon;
+		charPtr = checkArg(argc,&args, argv, flagChar);
+		if ((colon = strchr(charPtr,':'))) 
+                    {
+                    /* we have hostName:port */
+                    *colon++ = '\0';
+                    hostName = XtNewString(charPtr);
+                    port = atoi(colon);
+                    }
+		else
+		if ( *charPtr == '/' || *charPtr == '.')
+                    /* it looks like a filename */
+		    socketName = XtNewString(charPtr);
+		else 
+		    /* must be a number */
+		    port = atoi(charPtr);
+		}
+            else
+#endif
                              
             if (flagChar == 'f')    /***  File - Mode  ***/
                 {
@@ -325,7 +444,7 @@ char **env;
 		    }
                 }
 	    else
-		fprintf(stderr, "Wafe(main): Unknown Argument, ignoring!\n");
+		wafeWarn("main", "Unknown Argument, ignoring",NULL,NULL,NULL);
 	    }
 	else
 	    {
@@ -335,10 +454,22 @@ char **env;
 	    }
 	args++; 
 	}
-
     *cargvCount = NULL;
 
+#ifdef SOCKET_CONNECTION
+    if (port || socketName) 
+	{
+	int fd = hostName ?
+	    tcpHostPortConnection(hostName,port) :
+		port ? 
+		    tcpPortConnection(port) : 
+		    unixStreamConnection(socketName);
 
+	dup2(fd, fileno(stdin));
+	dup2(fd, fileno(stdout));
+	close(fd);
+	}
+#endif
 
     /* Check that everybody has got his args */
 
@@ -395,9 +526,15 @@ char **env;
     /* Must initialize here, so that Xt can strip off its arguments 
      * and application gets reduced argument list (frontend mode)
      */
-
+#ifndef PRER6
+    wafeTopLevel = XtVaOpenApplication(&wafeAppContext, wafeAppClass, NULL, 0, 
+				       &cargc, cargv, NULL, 
+				       applicationShellWidgetClass,
+				       NULL); 
+#else
     wafeTopLevel = XtVaAppInitialize(&wafeAppContext, wafeAppClass, NULL, 0, 
 				 &cargc, cargv, NULL, NULL); 
+#endif
 
     MOTIF_EDITRES_HANDLER(wafeTopLevel);
 
@@ -437,7 +574,7 @@ char **env;
           */
 
 	if (iSocketPair(sp,0) == -1)
-	    wafeError("main", "Couldn't create socket for stdin", NULL);
+	    wafeFatal("main", "Couldn't create socket for stdin", NULL);
 /*
  printf("sp %d %d %d %d\n",sp[0],sp[1],sp[2],sp[3]);
  */
@@ -447,7 +584,7 @@ char **env;
 	 */
 
 	if (iSocketPair(cp,1) == -1)
-	    wafeError("main", "Couldn't create socket for tunnel", NULL);
+	    wafeFatal("main", "Couldn't create socket for tunnel", NULL);
 
 /*
  printf("cp %d %d %d %d\n",cp[0],cp[1],cp[2],cp[3]);
@@ -466,7 +603,7 @@ char **env;
           */
 
 	if (iSocketPair(ep,1) == -1)
-	    wafeError("main", "Couldn't create socket for stderr", NULL);
+	    wafeFatal("main", "Couldn't create socket for stderr", NULL);
 
 /*
    printf("ep %d %d %d %d\n",ep[0],ep[1],ep[2],ep[3]);
@@ -502,11 +639,11 @@ char **env;
 	    cargv[0] = opt_p ? applicationName : applicationName+1;
 	    execvp(cargv[0], cargv);   /* execute the client-program  */  
    
-	    wafeError("main", "Cannot execute <%s>", cargv[0]);
+	    wafeFatal("main", "Cannot execute <%s>", cargv[0]);
 	    }
 
 	if (wafeClientPid == -1)
-	    wafeError("main", "Couldn't fork", NULL);
+	    wafeFatal("main", "Couldn't fork", NULL);
 
 /*
           close(sp[1]);
@@ -534,7 +671,7 @@ char **env;
 
      /* Now check for invalid combination of modes */
     if ((directMode) && (fileMode))
-	wafeError("main","Invalid combination direct-mode and file-mode",
+	wafeFatal("main","Invalid combination direct-mode and file-mode",
 		  NULL);
 
     /* Now set up interrup handler, initalize wafe packages, set
@@ -548,7 +685,7 @@ char **env;
 
     if (fileMode)
 	if (Tcl_EvalFile(wafeInterpreter, wafeScriptName) == TCL_ERROR)
-	    wafeError("file evaluation", "%s", wafeInterpreter->result);
+	    wafeFatal("file evaluation", "%s", wafeInterpreter->result);
 
 #ifdef RDD
     rddAppMainLoop(wafeAppContext);
